/********************************************************/
/* CMPSCI 603 vision interface to .pgm format (for xv)  */
/* Mar 1998  Justus Piater/Rod Grupen                   */
/*                                                      */
/* read_image(), remap(), write_image()                 */
/********************************************************/
#include <stdlib.h>
#include <stdio.h>

#include "pgm_interface.h"

/* remap range of intensities in h IMAGE into 0-255 for .pgm output */
re_map(img, min, max)
IMAGE *img;
int min, max;
{
  int x, y;
  double ratio;

  ratio = (double) 255 / (double)(max-min);
  printf("min = %d max = %d remapping to 0-255 (ratio = %lf)\n",
	 min, max, ratio);
  img->maxval = 255;
  for (x=0;x<img->width; ++x) {
    for (y=0; y<img->height; ++y) {
	img->image[y*img->width+x] = 
	  (int) ((img->image[y*img->width+x] - min) * ratio);
    }
  }
}

/* Reads an .pgm file into an IMAGE structure and returns a */
/* pointer to the IMAGE --- use free_image() to deallocate  */
IMAGE *read_image(filename)
char *filename;
{
  IMAGE *img;
  FILE *fp, *fopen();
  unsigned char *tmp;
  int x, y, nbytes;

  if ( !(fp = fopen(filename, "r")) ) {
    printf ("Cannot open %s\n", filename);
    return NULL;
  }
  img = (IMAGE *) malloc(sizeof(IMAGE));
  if (fscanf (fp, "P5%d%d%d\n", &img->width,&img->height,&img->maxval) != 3) {
    printf ("%s: Not a PGM file - or maybe a commented PGM file\n", filename);
    return NULL;
  }
  nbytes = img->width * img->height;
  if (!(tmp = (unsigned char *) malloc(nbytes))) {
    printf ("Cannot allocate %d bytes for %s\n", nbytes, filename);
    return NULL;
  }
  if (fread (tmp, 1, nbytes, fp) != nbytes) {
    perror ("Error reading PGMimage");
    return NULL;
  }
  nbytes *= sizeof(int);
  if (!(img->image = (int *) malloc(nbytes))) {
    printf ("Cannot allocate %d bytes for %s\n", nbytes, filename);
    return NULL;
  }
  for (x=0; x<(img->width); ++x) {
    for (y=0; y<(img->height); ++y) {
      img->image[y*(img->width)+x] = (int) tmp[y*(img->width)+x];
    }
  }
  free(tmp);
  fclose (fp);
  return(img);
}

write_image(img, filename)
IMAGE *img;
char *filename;
{
  FILE *fp, *fopen();
  unsigned char *tmp;
  int x, y, nbytes;

  if ( !(fp = fopen(filename, "w")) ) {
    printf ("Cannot open %s\n", filename);
    /*    return NULL; */
  }
  fprintf(fp,"P5 %d %d %d\n", img->width, img->height, img->maxval);
  
  nbytes = img->width * img->height;
  if (!(tmp = (unsigned char *) malloc(nbytes))) {
    printf ("Cannot allocate %d bytes for %s\n", nbytes, filename);
    /*    return NULL; */
  }
  for (x=0; x<(img->width); ++x) {
    for (y=0; y<(img->height); ++y) {
      tmp[y*(img->width)+x] = (unsigned char) img->image[y*(img->width)+x];
    }
  }
  if (fwrite (tmp, 1, nbytes, fp) != nbytes) {
    perror ("Error writing IMAGE");
    /*     return NULL; */
  }
  free(tmp);
  fclose (fp);
}

/* Frees memory allocated by read_image() */
free_image(img)
IMAGE *img;
{
  free(img->image);
  free(img);
}
